@@ -19,6 +19,7 @@ from uuid import uuid4
 import random
 from hashlib import md5
 from collections import defaultdict
+import os
 
 from swiftclient import client
 
@@ -45,8 +46,7 @@ class TestObjectHandoff(ReplProbeTest):
         opart, onodes = self.object_ring.get_nodes(
             self.account, container, obj)
         onode = onodes[0]
-        kill_server((onode['ip'], onode['port']),
-                    self.ipport2server, self.pids)
+        kill_server((onode['ip'], onode['port']), self.ipport2server)
 
         # Create container/obj (goes to two primary servers and one handoff)
         client.put_object(self.url, self.token, container, obj, 'VERIFY')
@@ -58,8 +58,7 @@ class TestObjectHandoff(ReplProbeTest):
         # Kill other two container/obj primary servers
         #   to ensure GET handoff works
         for node in onodes[1:]:
-            kill_server((node['ip'], node['port']),
-                        self.ipport2server, self.pids)
+            kill_server((node['ip'], node['port']), self.ipport2server)
 
         # Indirectly through proxy assert we can get container/obj
         odata = client.get_object(self.url, self.token, container, obj)[-1]
@@ -69,8 +68,7 @@ class TestObjectHandoff(ReplProbeTest):
 
         # Restart those other two container/obj primary servers
         for node in onodes[1:]:
-            start_server((node['ip'], node['port']),
-                         self.ipport2server, self.pids)
+            start_server((node['ip'], node['port']), self.ipport2server)
 
         # We've indirectly verified the handoff node has the container/object,
         #   but let's directly verify it.
@@ -82,6 +80,22 @@ class TestObjectHandoff(ReplProbeTest):
             raise Exception('Direct object GET did not return VERIFY, instead '
                             'it returned: %s' % repr(odata))
 
+        # drop a tempfile in the handoff's datadir, like it might have
+        # had if there was an rsync failure while it was previously a
+        # primary
+        handoff_device_path = self.device_dir('object', another_onode)
+        data_filename = None
+        for root, dirs, files in os.walk(handoff_device_path):
+            for filename in files:
+                if filename.endswith('.data'):
+                    data_filename = filename
+                    temp_filename = '.%s.6MbL6r' % data_filename
+                    temp_filepath = os.path.join(root, temp_filename)
+        if not data_filename:
+            self.fail('Did not find any data files on %r' %
+                      handoff_device_path)
+        open(temp_filepath, 'w')
+
         # Assert container listing (via proxy and directly) has container/obj
         objs = [o['name'] for o in
                 client.get_container(self.url, self.token, container)[1]]
@@ -97,8 +111,7 @@ class TestObjectHandoff(ReplProbeTest):
                     (cnode['ip'], cnode['port']))
 
         # Bring the first container/obj primary server back up
-        start_server((onode['ip'], onode['port']),
-                     self.ipport2server, self.pids)
+        start_server((onode['ip'], onode['port']), self.ipport2server)
 
         # Assert that it doesn't have container/obj yet
         try:
@@ -134,6 +147,20 @@ class TestObjectHandoff(ReplProbeTest):
             raise Exception('Direct object GET did not return VERIFY, instead '
                             'it returned: %s' % repr(odata))
 
+        # and that it does *not* have a temporary rsync dropping!
+        found_data_filename = False
+        primary_device_path = self.device_dir('object', onode)
+        for root, dirs, files in os.walk(primary_device_path):
+            for filename in files:
+                if filename.endswith('.6MbL6r'):
+                    self.fail('Found unexpected file %s' %
+                              os.path.join(root, filename))
+                if filename == data_filename:
+                    found_data_filename = True
+        self.assertTrue(found_data_filename,
+                        'Did not find data file %r on %r' % (
+                            data_filename, primary_device_path))
+
         # Assert the handoff server no longer has container/obj
         try:
             direct_client.direct_get_object(
@@ -146,8 +173,7 @@ class TestObjectHandoff(ReplProbeTest):
 
         # Kill the first container/obj primary server again (we have two
         #   primaries and the handoff up now)
-        kill_server((onode['ip'], onode['port']),
-                    self.ipport2server, self.pids)
+        kill_server((onode['ip'], onode['port']), self.ipport2server)
 
         # Delete container/obj
         try:
@@ -184,8 +210,7 @@ class TestObjectHandoff(ReplProbeTest):
                     (cnode['ip'], cnode['port']))
 
         # Restart the first container/obj primary server again
-        start_server((onode['ip'], onode['port']),
-                     self.ipport2server, self.pids)
+        start_server((onode['ip'], onode['port']), self.ipport2server)
 
         # Assert it still has container/obj
         direct_client.direct_get_object(
