@@ -256,9 +256,9 @@ def write_fake_ring(path, *devs):
     Pretty much just a two node, two replica, 2 part power ring...
     """
     dev1 = {'id': 0, 'zone': 0, 'device': 'sda1', 'ip': '127.0.0.1',
-            'port': 6000}
+            'port': 6200}
     dev2 = {'id': 0, 'zone': 0, 'device': 'sdb1', 'ip': '127.0.0.1',
-            'port': 6000}
+            'port': 6200}
 
     dev1_updates, dev2_updates = devs or ({}, {})
 
@@ -278,7 +278,7 @@ class FabricatedRing(Ring):
     your tests needs.
     """
 
-    def __init__(self, replicas=6, devices=8, nodes=4, port=6000,
+    def __init__(self, replicas=6, devices=8, nodes=4, port=6200,
                  part_power=4):
         self.devices = devices
         self.nodes = nodes
@@ -1052,32 +1052,3 @@ def mocked_http_conn(*args, **kwargs):
 
 def make_timestamp_iter():
     return iter(Timestamp(t) for t in itertools.count(int(time.time())))
-
-
-def encode_frag_archive_bodies(policy, body):
-    """
-    Given a stub body produce a list of complete frag_archive bodies as
-    strings in frag_index order.
-
-    :param policy: a StoragePolicy instance, with policy_type EC_POLICY
-    :param body: a string, the body to encode into frag archives
-
-    :returns: list of strings, the complete frag_archive bodies for the given
-              plaintext
-    """
-    segment_size = policy.ec_segment_size
-    # split up the body into buffers
-    chunks = [body[x:x + segment_size]
-              for x in range(0, len(body), segment_size)]
-    # encode the buffers into fragment payloads
-    fragment_payloads = []
-    for chunk in chunks:
-        fragments = policy.pyeclib_driver.encode(chunk)
-        if not fragments:
-            break
-        fragment_payloads.append(fragments)
-
-    # join up the fragment payloads per node
-    ec_archive_bodies = [''.join(frags)
-                         for frags in zip(*fragment_payloads)]
-    return ec_archive_bodies
